#include "mesh.h"

CMesh::CMesh(std::string& filename)
{
	loadObjFile(filename);

	m_transform = gmMatrix4::identity();
}

CMesh::~CMesh()
{
	clear();
}

void CMesh::addRotation(double angle, gmVector3& _axis)
{
	gmMatrix4 rotation = gmMatrix4::rotate(angle, _axis);
	m_transform *= rotation;
}

void CMesh::addTranslation(gmVector3& _translation)
{
	gmMatrix4 translation = gmMatrix4::translate(_translation.x, _translation.y, _translation.z);
	m_transform *= translation;
}

void CMesh::clear()
{
	m_vectorVertices.clear();
	m_vectorFaces.clear();
	m_vectorNormals.clear();
	m_vectorTextureCoordinates.clear();
}

void CMesh::draw() const
{
	//setDiffuseMaterial(0.99f, 0.99f, 0.99f);
	
	float transform[16]; 
	m_transform.copyTo(transform);
	
	glPushMatrix();
	glMultMatrixf(transform);

	glBegin(GL_TRIANGLES);
	for (size_t iFaces = 0; iFaces < m_vectorFaces.size(); iFaces ++)
	{
		glNormal3d(m_vectorNormals[m_vectorFaces[iFaces].n1].x, m_vectorNormals[m_vectorFaces[iFaces].n1].y, m_vectorNormals[m_vectorFaces[iFaces].n1].z);
		glVertex3d(m_vectorVertices[m_vectorFaces[iFaces].v1].x, m_vectorVertices[m_vectorFaces[iFaces].v1].y, m_vectorVertices[m_vectorFaces[iFaces].v1].z);

		// define second point of the triangle, including texture coordinates and normal
		glNormal3d(m_vectorNormals[m_vectorFaces[iFaces].n2].x, m_vectorNormals[m_vectorFaces[iFaces].n2].y, m_vectorNormals[m_vectorFaces[iFaces].n2].z);
		glVertex3d(m_vectorVertices[m_vectorFaces[iFaces].v2].x, m_vectorVertices[m_vectorFaces[iFaces].v2].y, m_vectorVertices[m_vectorFaces[iFaces].v2].z);

		// define third point of the triangle, including texture coordinates and normal
		glNormal3d(m_vectorNormals[m_vectorFaces[iFaces].n3].x, m_vectorNormals[m_vectorFaces[iFaces].n3].y, m_vectorNormals[m_vectorFaces[iFaces].n3].z);
		glVertex3d(m_vectorVertices[m_vectorFaces[iFaces].v3].x, m_vectorVertices[m_vectorFaces[iFaces].v3].y, m_vectorVertices[m_vectorFaces[iFaces].v3].z);
	
	}
	glEnd();

	glPopMatrix();

	// for debugging: wireframe mode
	/*glDisable(GL_LIGHTING);
	glColor3f(0.0f, 1.0f, 0.0f);
	glBegin(GL_LINES);
	for (size_t iFaces = 0; iFaces < m_vectorFaces.size(); iFaces ++)
	{
		gmVector3 tmp(m_vectorVertices[m_vectorFaces[iFaces].v1] + m_vectorVertices[m_vectorFaces[iFaces].v2] + m_vectorVertices[m_vectorFaces[iFaces].v3]);
		tmp /= 3.0;
		glVertex3d( tmp.x,  tmp.y,  tmp.z);
		tmp += m_vectorFaces[iFaces].normal / 2.0;
		glVertex3d( tmp.x,  tmp.y,  tmp.z);
	}
	glEnd();
	glEnable(GL_LIGHTING);
	*/
}

const std::vector<CMesh::Edge>& CMesh::getEdges() const
{
	return m_vectorEdges;
}

std::vector<CMesh::Face>& CMesh::getFaces()
{
	return m_vectorFaces;
}

const std::vector<CMesh::Vertex>& CMesh::getVertices() const
{
	return m_vectorVertices;
}

const std::vector<CMesh::Normal>& CMesh::getNormals() const
{
	return m_vectorNormals;
}

const std::vector<CMesh::TextureCoordinate>& CMesh::getTextureCoordinates() const
{
	return m_vectorTextureCoordinates;
}

void CMesh::setDiffuseMaterial(GLfloat red, GLfloat green, GLfloat blue) const
{
	GLfloat color[] = {red, green, blue, 1.0};
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
}

void CMesh::setSpecularMaterial(GLfloat red, GLfloat green, GLfloat blue) const
{
	GLfloat color[] = {red, green, blue, 1.0};

	glMaterialfv(GL_FRONT, GL_AMBIENT, color);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, color);
	glMaterialfv(GL_FRONT, GL_SPECULAR, color);
	glMaterialf(GL_FRONT, GL_SHININESS, 50);
}
