/******************************************************************************
 * volume_shadow.cpp 
 * implements the construction and drawing of shadow volumes from triangulated surface meshes 
 * Copyright (C) 2010 Marc Gissler <gisslerm@informatik.uni-freiburg.de>
 ******************************************************************************/

#include "volume_shadow.h"

#include "scene.h"
#include "lightmanager.h"

#include <set>

//#define BRUTE_FORCE_SHADOW_VOLUMES

CVolumeShadow::CVolumeShadow()
{
	//m_color = ShadowColor(0.5, 0.5, 0.5, 0.2);
	//clear();
}

CVolumeShadow::~CVolumeShadow()
{
	clear();
}

void CVolumeShadow::build(int lightId, gmVector3& lightPos)
{
	std::map<int, CShadowData*>::iterator mapIter;
	mapIter = m_mapLightAndData.find(lightId);

	if(mapIter == m_mapLightAndData.end())
	{
		CVolumeShadowData* data = new CVolumeShadowData();
		mapIter = m_mapLightAndData.insert(std::pair<int, CShadowData*>(lightId, data)).first;
	}

	CVolumeShadowData* data = static_cast<CVolumeShadowData*>((*mapIter).second);
	std::vector<ShadowFace>& vectorShadowFaces = data->m_vectorShadowFaces;
	std::vector<ShadowVertex>& vectorShadowVertices = data->m_vectorShadowVertices;

	// clear the previous shadow volumes
	vectorShadowFaces.clear();

	std::list<CMesh*>& listMeshes = CScene::getScene()->getMeshes();
	std::list<CMesh*>::iterator meshIter;
	for(meshIter = listMeshes.begin(); meshIter != listMeshes.end(); ++meshIter)
	{
		CMesh* mesh = *meshIter;

		gmMatrix4& transf = mesh->m_transform;

		const std::vector<CMesh::Vertex>& vecVertices = mesh->getVertices();
		std::vector<CMesh::Face>& vecFaces = mesh->getFaces();
		const std::vector<CMesh::Edge>& vecEdges = mesh->getEdges();
		
		// TODO: put shadow-volume-generating code here

	}
}

void CVolumeShadow::clear()
{
	std::map<int, CShadowData*>::iterator mapIter;
	for(mapIter = m_mapLightAndData.begin(); mapIter != m_mapLightAndData.end(); ++mapIter)
	{
		CVolumeShadowData* data = static_cast<CVolumeShadowData*>(mapIter->second);
		delete data;
	}
	m_mapLightAndData.clear();
}

void CVolumeShadow::m_drawShadowVolumes(int lightId)
{
	std::map<int, CShadowData*>::iterator mapIter;
	mapIter = m_mapLightAndData.find(lightId);

	if(mapIter == m_mapLightAndData.end())
		return; // error: should be in here!
	
	CVolumeShadowData* data = static_cast<CVolumeShadowData*>((*mapIter).second);
	std::vector<ShadowFace>& vectorShadowFaces = data->m_vectorShadowFaces;

	// draw the shadow primitives
	glBegin(GL_TRIANGLES);
	for(size_t i = 0; i < vectorShadowFaces.size(); ++i)
	{
		ShadowFace& face = vectorShadowFaces[i];

		//note: give the vertices as 4D-vector to be able to determine the W-coordinate (some of the vertices are located at inifinity (w = 0) )
		glVertex4f( (GLfloat) (face.v1.x), (GLfloat) (face.v1.y), (GLfloat) (face.v1.z), (GLfloat) face.v1.w);
		glVertex4f( (GLfloat) (face.v2.x), (GLfloat) (face.v2.y), (GLfloat) (face.v2.z), (GLfloat) face.v2.w);
		glVertex4f( (GLfloat) (face.v3.x), (GLfloat) (face.v3.y), (GLfloat) (face.v3.z), (GLfloat) face.v3.w);
	}
	glEnd();
}

void CVolumeShadow::drawSceneWithShadows()
{
	CScene* pScene = CScene::getScene();
	std::map<int, CShadowData*>::iterator mapIter;

	for(mapIter = m_mapLightAndData.begin(); mapIter != m_mapLightAndData.end(); ++mapIter)
	{
		int lightId = mapIter->first;
		glEnable(GL_LIGHT0+lightId);
	}

	// render ambient intensities
	CLightManager::getLightManager()->setAmbientExclusively();
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	pScene->drawMeshes();
	CLightManager::getLightManager()->setAllIntensities();

	for(mapIter = m_mapLightAndData.begin(); mapIter != m_mapLightAndData.end(); ++mapIter)
	{
		int lightId = mapIter->first;
		glDisable(GL_LIGHT0+lightId);
	}

  // TODO: Render your shadow volume; you can use the code below

	//// debug: render shadow volumes into color buffer
	//{
	//	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	//	glPushAttrib(GL_ALL_ATTRIB_BITS);
	//	glDisable(GL_DEPTH_TEST);
	//	glDisable(GL_LIGHTING);
	//	glDisable(GL_CULL_FACE);
	//	glColor4f(0.5f, 0.5f, 0.5f, 0.5f);
	//	//glEnable(GL_BLEND);
	//	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//	m_drawShadowVolumes(0);
	//	glEnable(GL_DEPTH_TEST);
	//	//glDisable(GL_BLEND);
	//	glEnable(GL_CULL_FACE);
	//	glPopAttrib();
	//	return;
	//}

}
